@@ -28,21 +28,21 @@
 #endif
 #include <rx\rx.h>
 #include <rx\rx_null.h>
+#include <rx\rxstat.h>
 #include <WINNT/syscfg.h>
 #include <WINNT/afsreg.h>
+#include <afs\afscbint.h>
 
 #include "smb.h"
 #include "cm_rpc.h"
 #include "lanahelper.h"
 #include <strsafe.h>
 #include "cm_memmap.h"
+#include "msrpc.h"
 #ifdef DEBUG
 #include <crtdbg.h>
 #endif
 
-extern int RXAFSCB_ExecuteRequest(struct rx_call *z_call);
-extern int RXSTATS_ExecuteRequest(struct rx_call *z_call);
-
 extern afs_uint32 cryptall;
 extern afs_uint32 cm_anonvldb;
 extern int cm_enableServerLocks;
@@ -86,8 +86,8 @@ char cm_HostName[200];
 long cm_HostAddr;
 unsigned short cm_callbackport = CM_DEFAULT_CALLBACKPORT;
 
-char cm_NetbiosName[MAX_NB_NAME_LENGTH] = "";
-clientchar_t cm_NetbiosNameC[MAX_NB_NAME_LENGTH] = _C("");
+char cm_NetbiosName[MAX_NB_NAME_LENGTH] = "NOT.YET.SET";
+clientchar_t cm_NetbiosNameC[MAX_NB_NAME_LENGTH] = _C("NOT.YET.SET");
 
 char cm_CachePath[MAX_PATH];
 DWORD cm_ValidateCache = 1;
@@ -240,159 +240,6 @@ void afsd_ForceTrace(BOOL flush)
     CloseHandle(handle);
 }
 
-static void
-configureBackConnectionHostNames(void)
-{
-    /* On Windows XP SP2, Windows 2003 SP1, and all future Windows operating systems
-     * there is a restriction on the use of SMB authentication on loopback connections.
-     * There are two work arounds available:
-     * 
-     *   (1) We can disable the check for matching host names.  This does not
-     *   require a reboot:
-     *   [HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\Lsa]
-     *     "DisableLoopbackCheck"=dword:00000001
-     *
-     *   (2) We can add the AFS SMB/CIFS service name to an approved list.  This
-     *   does require a reboot:
-     *   [HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\Lsa\MSV1_0]
-     *     "BackConnectionHostNames"=multi-sz
-     *
-     * The algorithm will be:
-     *   (1) Check to see if cm_NetbiosName exists in the BackConnectionHostNames list
-     *   (2a) If not, add it to the list.  (This will not take effect until the next reboot.)
-     *   (2b1)    and check to see if DisableLoopbackCheck is set.
-     *   (2b2)    If not set, set the DisableLoopbackCheck value to 0x1 
-     *   (2b3)                and create HKLM\SOFTWARE\OpenAFS\Client  UnsetDisableLoopbackCheck
-     *   (2c) else If cm_NetbiosName exists in the BackConnectionHostNames list,
-     *             check for the UnsetDisableLoopbackCheck value.  
-     *             If set, set the DisableLoopbackCheck flag to 0x0 
-     *             and delete the UnsetDisableLoopbackCheck value
-     *
-     * Starting in Longhorn Beta 1, an entry in the BackConnectionHostNames value will
-     * force Windows to use the loopback authentication mechanism for the specified 
-     * services.
-     */
-    HKEY hkLsa;
-    HKEY hkMSV10;
-    HKEY hkClient;
-    DWORD dwType;
-    DWORD dwSize, dwAllocSize;
-    DWORD dwValue;
-    PBYTE pHostNames = NULL, pName = NULL;
-    BOOL  bNameFound = FALSE;   
-
-    if ( RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
-                       "SYSTEM\\CurrentControlSet\\Control\\Lsa\\MSV1_0",
-                       0,
-                       KEY_READ|KEY_WRITE,
-                       &hkMSV10) == ERROR_SUCCESS )
-    {
-        if ((RegQueryValueEx( hkMSV10, "BackConnectionHostNames", 0, 
-			     &dwType, NULL, &dwAllocSize) == ERROR_SUCCESS) &&
-            (dwType == REG_MULTI_SZ)) 
-        {
-	    dwAllocSize += 1 /* in case the source string is not nul terminated */
-		+ strlen(cm_NetbiosName) + 2;
-	    pHostNames = malloc(dwAllocSize);
-	    dwSize = dwAllocSize;
-            if (RegQueryValueEx( hkMSV10, "BackConnectionHostNames", 0, &dwType, 
-				 pHostNames, &dwSize) == ERROR_SUCCESS) 
-            {
-		for (pName = pHostNames; 
-		     (pName - pHostNames < (int) dwSize) && *pName ; 
-		     pName += strlen(pName) + 1)
-		{
-		    if ( !stricmp(pName, cm_NetbiosName) ) {
-			bNameFound = TRUE;
-			break;
-		    }   
-		}
-	    }
-        }
-             
-        if ( !bNameFound ) {
-            size_t size = strlen(cm_NetbiosName) + 2;
-            if ( !pHostNames ) {
-                pHostNames = malloc(size);
-		pName = pHostNames;
-            }
-            StringCbCopyA(pName, size, cm_NetbiosName);
-            pName += size - 1;
-            *pName = '\0';  /* add a second nul terminator */
-
-            dwType = REG_MULTI_SZ;
-	    dwSize = pName - pHostNames + 1;
-            RegSetValueEx( hkMSV10, "BackConnectionHostNames", 0, dwType, pHostNames, dwSize);
-
-            if ( RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
-                               "SYSTEM\\CurrentControlSet\\Control\\Lsa",
-                               0,
-                               KEY_READ|KEY_WRITE,
-                               &hkLsa) == ERROR_SUCCESS )
-            {
-                dwSize = sizeof(DWORD);
-                if ( RegQueryValueEx( hkLsa, "DisableLoopbackCheck", 0, &dwType, (LPBYTE)&dwValue, &dwSize) != ERROR_SUCCESS ||
-                     dwValue == 0 ) {
-                    dwType = REG_DWORD;
-                    dwSize = sizeof(DWORD);
-                    dwValue = 1;
-                    RegSetValueEx( hkLsa, "DisableLoopbackCheck", 0, dwType, (LPBYTE)&dwValue, dwSize);
-
-                    if (RegCreateKeyEx( HKEY_LOCAL_MACHINE, 
-                                        AFSREG_CLT_OPENAFS_SUBKEY,
-                                        0,
-                                        NULL,
-                                        REG_OPTION_NON_VOLATILE,
-                                        KEY_READ|KEY_WRITE,
-                                        NULL,
-                                        &hkClient,
-                                        NULL) == ERROR_SUCCESS) {
-
-                        dwType = REG_DWORD;
-                        dwSize = sizeof(DWORD);
-                        dwValue = 1;
-                        RegSetValueEx( hkClient, "RemoveDisableLoopbackCheck", 0, dwType, (LPBYTE)&dwValue, dwSize);
-                        RegCloseKey(hkClient);
-                    }
-                    RegCloseKey(hkLsa);
-                }
-            }
-        } else {
-            if (RegCreateKeyEx( HKEY_LOCAL_MACHINE, 
-                                AFSREG_CLT_OPENAFS_SUBKEY,
-                                0,
-                                NULL,
-                                REG_OPTION_NON_VOLATILE,
-                                KEY_READ|KEY_WRITE,
-                                NULL,
-                                &hkClient,
-                                NULL) == ERROR_SUCCESS) {
-
-                dwSize = sizeof(DWORD);
-                if ( RegQueryValueEx( hkClient, "RemoveDisableLoopbackCheck", 0, &dwType, (LPBYTE)&dwValue, &dwSize) == ERROR_SUCCESS &&
-                     dwValue == 1 ) {
-                    if ( RegOpenKeyEx( HKEY_LOCAL_MACHINE, 
-                                       "SYSTEM\\CurrentControlSet\\Control\\Lsa",
-                                       0,
-                                       KEY_READ|KEY_WRITE,
-                                       &hkLsa) == ERROR_SUCCESS )
-                    {
-                        RegDeleteValue(hkLsa, "DisableLoopbackCheck");
-                        RegCloseKey(hkLsa);
-                    }
-                }
-                RegDeleteValue(hkClient, "RemoveDisableLoopbackCheck");
-                RegCloseKey(hkClient);
-            }
-        }
-        RegCloseKey(hkMSV10);
-    }
-
-    if (pHostNames)
-	free(pHostNames);
-}
-
-
 static void afsd_InitServerPreferences(void)
 {
     HKEY hkPrefs = 0;
@@ -456,7 +303,10 @@ static void afsd_InitServerPreferences(void)
             tsp = cm_FindServer(&saddr, CM_SERVER_VLDB);
             if ( tsp )		/* an existing server - ref count increased */
             {
-                tsp->ipRank = (USHORT)dwRank; /* no need to protect by mutex*/
+                lock_ObtainMutex(&tsp->mx);
+                tsp->ipRank = (USHORT)dwRank;
+                tsp->flags |= CM_SERVERFLAG_PREF_SET;
+                lock_ReleaseMutex(&tsp->mx);
 
                 /* set preferences for an existing vlserver */
                 cm_ChangeRankCellVLServer(tsp);
@@ -464,8 +314,11 @@ static void afsd_InitServerPreferences(void)
             }
             else	/* add a new server without a cell */
             {
-                tsp = cm_NewServer(&saddr, CM_SERVER_VLDB, NULL, CM_FLAG_NOPROBE); /* refcount = 1 */
+                tsp = cm_NewServer(&saddr, CM_SERVER_VLDB, NULL, NULL, CM_FLAG_NOPROBE); /* refcount = 1 */
+                lock_ObtainMutex(&tsp->mx);
                 tsp->ipRank = (USHORT)dwRank;
+                tsp->flags |= CM_SERVERFLAG_PREF_SET;
+                lock_ReleaseMutex(&tsp->mx);
             }
         }
 
@@ -523,7 +376,10 @@ static void afsd_InitServerPreferences(void)
             tsp = cm_FindServer(&saddr, CM_SERVER_FILE);
             if ( tsp )		/* an existing server - ref count increased */
             {
-                tsp->ipRank = (USHORT)dwRank; /* no need to protect by mutex*/
+                lock_ObtainMutex(&tsp->mx);
+                tsp->ipRank = (USHORT)dwRank;
+                tsp->flags |= CM_SERVERFLAG_PREF_SET;
+                lock_ReleaseMutex(&tsp->mx);
 
                 /* find volumes which might have RO copy 
                 /* on server and change the ordering of 
@@ -534,8 +390,11 @@ static void afsd_InitServerPreferences(void)
             }
             else	/* add a new server without a cell */
             {
-                tsp = cm_NewServer(&saddr, CM_SERVER_FILE, NULL, CM_FLAG_NOPROBE); /* refcount = 1 */
+                tsp = cm_NewServer(&saddr, CM_SERVER_FILE, NULL, NULL, CM_FLAG_NOPROBE); /* refcount = 1 */
+                lock_ObtainMutex(&tsp->mx);
                 tsp->ipRank = (USHORT)dwRank;
+                tsp->flags |= CM_SERVERFLAG_PREF_SET;
+                lock_ReleaseMutex(&tsp->mx);
             }
         }
 
@@ -579,7 +438,48 @@ is_wow64(void)
  * AFSD Initialization
  */
 
-int afsd_InitCM(char **reasonP)
+static int
+afsd_InitRoot(char **reasonP)
+{
+    long code;
+    cm_req_t req;
+
+    cm_InitReq(&req);
+
+    if (cm_freelanceEnabled) {
+        cm_FakeRootFid(&cm_data.rootFid);
+    } else {
+	int attempts = 10;
+
+        osi_Log0(afsd_logp, "Loading Root Volume from cell");
+	do {
+	    code = cm_FindVolumeByName(cm_data.rootCellp, cm_rootVolumeName, cm_rootUserp,
+				       &req, CM_GETVOL_FLAG_CREATE, &cm_data.rootVolumep);
+	    afsi_log("cm_FindVolumeByName code %x root vol %x", code,
+		      (code ? (cm_volume_t *)-1 : cm_data.rootVolumep));
+	} while (code && --attempts);
+        if (code != 0) {
+            *reasonP = "can't find root volume in root cell";
+            return -1;
+        }
+
+        /* compute the root fid */
+        cm_SetFid(&cm_data.rootFid, cm_data.rootCellp->cellID, cm_GetROVolumeID(cm_data.rootVolumep), 1, 1);
+    }
+
+    code = cm_GetSCache(&cm_data.rootFid, &cm_data.rootSCachep, cm_rootUserp, &req);
+    afsi_log("cm_GetSCache code %x scache %x", code,
+             (code ? (cm_scache_t *)-1 : cm_data.rootSCachep));
+    if (code != 0) {
+        *reasonP = "unknown error";
+        return -1;
+    }
+
+    return 0;
+}
+
+int
+afsd_InitCM(char **reasonP)
 {
     osi_uid_t debugID;
     afs_uint64 cacheBlocks;
@@ -617,11 +517,14 @@ int afsd_InitCM(char **reasonP)
     int cm_SubnetMask[CM_MAXINTERFACE_ADDR];/* client's subnet mask in host order*/
     int cm_NetMtu[CM_MAXINTERFACE_ADDR];    /* client's MTU sizes */
     int cm_NetFlags[CM_MAXINTERFACE_ADDR];  /* network flags */
+    DWORD dwPriority;
 
     WSAStartup(0x0101, &WSAjunk);
 
     init_et_to_sys_error();
 
+    cm_utilsInit();
+
     /* setup osidebug server at RPC slot 1000 */
     osi_LongToUID(1000, &debugID);
     code = osi_InitDebug(&debugID);
@@ -666,6 +569,16 @@ int afsd_InitCM(char **reasonP)
         osi_panic(buf, __FILE__, __LINE__);
     }
 
+    dummyLen = sizeof(dwPriority);
+    code = RegQueryValueEx(parmKey, "PriorityClass", NULL, NULL,
+                            (BYTE *) &dwPriority, &dummyLen);
+    if (code != ERROR_SUCCESS || dwPriority == 0) {
+        dwPriority = HIGH_PRIORITY_CLASS;
+    }
+    if (dwPriority != GetPriorityClass(GetCurrentProcess()))
+        SetPriorityClass(GetCurrentProcess(), dwPriority);
+    afsi_log("PriorityClass 0x%x", GetPriorityClass(GetCurrentProcess()));
+
     dummyLen = sizeof(lockOrderValidation);
     code = RegQueryValueEx(parmKey, "LockOrderValidation", NULL, NULL,
                             (BYTE *) &lockOrderValidation, &dummyLen);
@@ -889,15 +802,15 @@ int afsd_InitCM(char **reasonP)
                             (LPBYTE) cm_mountRootC, &cm_mountRootCLen);
     if (code == ERROR_SUCCESS) {
         afsi_log("Mount root %S", cm_mountRootC);
-        cm_mountRootCLen = cm_ClientStrLen(cm_mountRootC);
+        cm_mountRootCLen = (DWORD)cm_ClientStrLen(cm_mountRootC);
     } else {
         cm_ClientStrCpy(cm_mountRootC, lengthof(cm_mountRootC), _C("/afs"));
-        cm_mountRootCLen = cm_ClientStrLen(cm_mountRootC);
+        cm_mountRootCLen = (DWORD)cm_ClientStrLen(cm_mountRootC);
         /* Don't log */
     }
 
     cm_ClientStringToFsString(cm_mountRootC, -1, cm_mountRoot, lengthof(cm_mountRoot));
-    cm_mountRootLen = cm_FsStrLen(cm_mountRoot);
+    cm_mountRootLen = (DWORD)cm_FsStrLen(cm_mountRoot);
 
     dummyLen = sizeof(buf);
     code = RegQueryValueEx(parmKey, "CachePath", NULL, &regType,
@@ -1287,6 +1200,14 @@ int afsd_InitCM(char **reasonP)
     } 
     afsi_log("CM FollowBackupPath is %u", cm_followBackupPath);
 
+    dummyLen = sizeof(DWORD);
+    code = RegQueryValueEx(parmKey, "PerFileAccessCheck", NULL, NULL,
+                           (BYTE *) &dwValue, &dummyLen);
+    if (code == ERROR_SUCCESS) {
+        cm_accessPerFileCheck = (int) dwValue;
+    } 
+    afsi_log("CM PerFileAccessCheck is %d", cm_accessPerFileCheck);
+
     RegCloseKey (parmKey);
 
     cacheBlocks = ((afs_uint64)cacheSize * 1024) / blockSize;
@@ -1316,9 +1237,6 @@ int afsd_InitCM(char **reasonP)
     cm_initParams.setTime = 0;
     cm_initParams.memCache = 1;
 
-    /* Ensure the AFS Netbios Name is registered to allow loopback access */
-    configureBackConnectionHostNames();
-
     /* init user daemon, and other packages */
     cm_InitUser();
 
@@ -1445,14 +1363,24 @@ int afsd_InitCM(char **reasonP)
     /* Initialize the RPC server for session keys */
     RpcInit();
 
+    /* Initialize the RPC server for pipe services */
+    MSRPC_Init();
+
     afsd_InitServerPreferences();
-    return 0;
+
+    code = afsd_InitRoot(reasonP);
+
+    return code;
 }
 
 int afsd_ShutdownCM(void)
 {
+    MSRPC_Shutdown();
+
     cm_ReleaseSCache(cm_data.rootSCachep);
 
+    cm_utilsCleanup();
+
     cm_shutdown = 1;
 
     return 0;
@@ -1460,44 +1388,6 @@ int afsd_ShutdownCM(void)
 
 int afsd_InitDaemons(char **reasonP)
 {
-    long code;
-    cm_req_t req;
-
-    cm_InitReq(&req);
-
-    /* this should really be in an init daemon from here on down */
-
-    if (!cm_freelanceEnabled) {
-	int attempts = 10;
-
-        osi_Log0(afsd_logp, "Loading Root Volume from cell");
-	do {
-	    code = cm_FindVolumeByName(cm_data.rootCellp, cm_rootVolumeName, cm_rootUserp,
-				       &req, CM_GETVOL_FLAG_CREATE, &cm_data.rootVolumep);
-	    afsi_log("cm_FindVolumeByName code %x root vol %x", code,
-		      (code ? (cm_volume_t *)-1 : cm_data.rootVolumep));
-	} while (code && --attempts);
-        if (code != 0) {
-            *reasonP = "can't find root volume in root cell";
-            return -1;
-        }
-    }
-
-    /* compute the root fid */
-    if (!cm_freelanceEnabled) {
-        cm_SetFid(&cm_data.rootFid, cm_data.rootCellp->cellID, cm_GetROVolumeID(cm_data.rootVolumep), 1, 1);
-    }
-    else
-        cm_FakeRootFid(&cm_data.rootFid);
-        
-    code = cm_GetSCache(&cm_data.rootFid, &cm_data.rootSCachep, cm_rootUserp, &req);
-    afsi_log("cm_GetSCache code %x scache %x", code,
-             (code ? (cm_scache_t *)-1 : cm_data.rootSCachep));
-    if (code != 0) {
-        *reasonP = "unknown error";
-        return -1;
-    }
-
     cm_InitDaemon(numBkgD);
     afsi_log("cm_InitDaemon complete");
 
