@@ -10,8 +10,6 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
-RCSID
-    ("$Header: /cvs/openafs/src/afs/FBSD/osi_file.c,v 1.13.14.1 2006/11/09 23:30:53 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -26,7 +24,7 @@ extern struct mount *afs_cacheVfsp;
 
 
 void *
-osi_UFSOpen(afs_int32 ainode)
+osi_UFSOpen(afs_dcache_id_t *ainode)
 {
     struct osi_file *afile;
     struct vnode *vp;
@@ -39,9 +37,9 @@ osi_UFSOpen(afs_int32 ainode)
     afile = (struct osi_file *)osi_AllocSmallSpace(sizeof(struct osi_file));
     AFS_GUNLOCK();
 #if defined(AFS_FBSD50_ENV)
-    code = VFS_VGET(afs_cacheVfsp, (ino_t) ainode, LK_EXCLUSIVE, &vp);
+    code = VFS_VGET(afs_cacheVfsp, (ino_t) ainode->ufs, LK_EXCLUSIVE, &vp);
 #else
-    code = VFS_VGET(afs_cacheVfsp, (ino_t) ainode, &vp);
+    code = VFS_VGET(afs_cacheVfsp, (ino_t) ainode->ufs, &vp);
 #endif
     AFS_GLOCK();
     if (code == 0 && vp->v_type == VNON)
@@ -50,7 +48,9 @@ osi_UFSOpen(afs_int32 ainode)
 	osi_FreeSmallSpace(afile);
 	osi_Panic("UFSOpen: igetinode failed");
     }
-#if defined(AFS_FBSD50_ENV)
+#if defined(AFS_FBSD80_ENV)
+    VOP_UNLOCK(vp, 0);
+#elif defined(AFS_FBSD50_ENV)
     VOP_UNLOCK(vp, 0, curthread);
 #else
     VOP_UNLOCK(vp, 0, curproc);
@@ -59,7 +59,7 @@ osi_UFSOpen(afs_int32 ainode)
     afile->size = VTOI(vp)->i_size;
     afile->offset = 0;
     afile->proc = NULL;
-    afile->inum = ainode;	/* for hint validity checking */
+    afile->inum = ainode->ufs;	/* for hint validity checking */
     return (void *)afile;
 }
 
@@ -71,7 +71,11 @@ afs_osi_Stat(register struct osi_file *afile, register struct osi_stat *astat)
     AFS_STATCNT(osi_Stat);
     MObtainWriteLock(&afs_xosi, 320);
     AFS_GUNLOCK();
-#if defined(AFS_FBSD50_ENV)
+#if defined(AFS_FBSD80_ENV)
+    vn_lock(afile->vnode, LK_EXCLUSIVE | LK_RETRY);
+    code = VOP_GETATTR(afile->vnode, &tvattr, afs_osi_credp, curthread);
+    VOP_UNLOCK(afile->vnode, 0);
+#elif defined(AFS_FBSD50_ENV)
     vn_lock(afile->vnode, LK_EXCLUSIVE | LK_RETRY, curthread);
     code = VOP_GETATTR(afile->vnode, &tvattr, afs_osi_credp, curthread);
     VOP_UNLOCK(afile->vnode, LK_EXCLUSIVE, curthread);
@@ -105,7 +109,7 @@ osi_UFSTruncate(register struct osi_file *afile, afs_int32 asize)
 {
     struct vattr tvattr;
     struct vnode *vp;
-    register afs_int32 code;
+    register afs_int32 code, glocked;
     AFS_STATCNT(osi_Truncate);
 
     MObtainWriteLock(&afs_xosi, 321);
@@ -115,8 +119,13 @@ osi_UFSTruncate(register struct osi_file *afile, afs_int32 asize)
      * have very slow truncates, even when the file is already
      * small enough.  Check now and save some time.
      */
-    AFS_GUNLOCK();
-#if defined(AFS_FBSD50_ENV)
+    glocked = ISAFS_GLOCK();
+    if (glocked)
+      AFS_GUNLOCK();
+#if defined(AFS_FBSD80_ENV)
+    vn_lock(vp, LK_EXCLUSIVE | LK_RETRY);
+    code = VOP_GETATTR(afile->vnode, &tvattr, afs_osi_credp, curthread);
+#elif defined(AFS_FBSD50_ENV)
     vn_lock(vp, LK_EXCLUSIVE | LK_RETRY, curthread);
     code = VOP_GETATTR(afile->vnode, &tvattr, afs_osi_credp, curthread);
 #else
@@ -135,12 +144,15 @@ osi_UFSTruncate(register struct osi_file *afile, afs_int32 asize)
 #endif
 
 out:
-#if defined(AFS_FBSD50_ENV)
+#if defined(AFS_FBSD80_ENV)
+    VOP_UNLOCK(vp, 0);
+#elif defined(AFS_FBSD50_ENV)
     VOP_UNLOCK(vp, LK_EXCLUSIVE, curthread);
 #else
     VOP_UNLOCK(vp, LK_EXCLUSIVE, curproc);
 #endif
-    AFS_GLOCK();
+    if (glocked)
+      AFS_GLOCK();
     MReleaseWriteLock(&afs_xosi);
     return code;
 }
@@ -244,8 +256,6 @@ afs_osi_MapStrategy(int (*aproc) (), register struct buf *bp)
 void
 shutdown_osifile(void)
 {
-    extern int afs_cold_shutdown;
-
     AFS_STATCNT(shutdown_osifile);
     if (afs_cold_shutdown) {
 	afs_osicred_initialized = 0;
