@@ -18,8 +18,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/uss/uss_vol.c,v 1.10.14.6 2009/03/27 15:47:32 shadow Exp $");
 
 #include "uss_vol.h"		/*Interface to this module */
 #include "uss_common.h"		/*Common definitions */
@@ -905,7 +903,9 @@ uss_vol_DeleteVol(char *a_volName, afs_int32 a_volID, char *a_servName,
 		  afs_int32 a_servID, char *a_partName, afs_int32  a_partID)
 {				/*uss_vol_DeleteVol */
 
+#ifdef USS_VOL_DB
     static char rn[] = "uss_vol_DeleteVol";	/*Routine name */
+#endif
     register afs_int32 code = 0;	/*Return code */
 
     /*
@@ -1087,7 +1087,7 @@ uss_vol_GetVolInfoFromMountPoint(char *a_mountpoint)
 	return (-1);
     }
     if (vldbEntry.nServers != 1) {
-	printf("s: Volume '%s' (ID %d) exists on multiple servers!!\n",
+	printf("%s: Volume '%s' (ID %d) exists on multiple servers!!\n",
 	       uss_whoami, uss_Volume, volID);
 	return (-1);
     }
