@@ -12,8 +12,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/vol/namei_ops.c,v 1.28.2.21 2009/03/24 13:36:38 shadow Exp $");
 
 #ifdef AFS_NAMEI_ENV
 #include <stdio.h>
@@ -100,7 +98,6 @@ emul_flock(int fd, int cmd)
 #define flock(f,c)      emul_flock(f,c)
 #endif
 
-extern char *volutil_PartitionName_r(int volid, char *buf, int buflen);
 int Testing=0;
 
 
@@ -196,22 +193,30 @@ static int GetFreeTag(IHandle_t * ih, int vno);
 static void
 namei_HandleToInodeDir(namei_t * name, IHandle_t * ih)
 {
-    char *tmp = name->n_base;
+    size_t offset;
 
     memset(name, '\0', sizeof(*name));
 
-    (void)volutil_PartitionName_r(ih->ih_dev, tmp, NAMEI_LCOMP_LEN);
-    tmp += VICE_PREFIX_SIZE;
-    tmp += ih->ih_dev > 25 ? 2 : 1;
-    *tmp = '/';
-    tmp++;
-    (void)strcpy(tmp, INODEDIR);
-    (void)strcpy(name->n_path, name->n_base);
+    /*
+     * Add the /vicepXX string to the start of name->n_base and then calculate
+     * offset as the number of bytes we know we added.
+     *
+     * FIXME: This embeds knowledge of the vice partition naming scheme and
+     * mapping from device numbers.  There needs to be an API that tells us
+     * this offset.
+     */
+    volutil_PartitionName_r(ih->ih_dev, name->n_base, sizeof(name->n_base));
+    offset = VICE_PREFIX_SIZE + (ih->ih_dev > 25 ? 2 : 1);
+    name->n_base[offset] = '/';
+    offset++;
+    strlcpy(name->n_base + offset, INODEDIR, sizeof(name->n_base) - offset);
+    strlcpy(name->n_path, name->n_base, sizeof(name->n_path));
 }
 
-#define addtoname(N, C) \
-do { \
-	 strcat((N)->n_path, "/"); strcat((N)->n_path, C); \
+#define addtoname(N, C)                                 \
+do {                                                    \
+    strlcat((N)->n_path, "/", sizeof((N)->n_path));     \
+    strlcat((N)->n_path, (C), sizeof((N)->n_path));     \
 } while(0)
 
 
@@ -222,10 +227,10 @@ namei_HandleToVolDir(namei_t * name, IHandle_t * ih)
 
     namei_HandleToInodeDir(name, ih);
     (void)int32_to_flipbase64(tmp, (int64_t) (ih->ih_vid & 0xff));
-    (void)strcpy(name->n_voldir1, tmp);
+    strlcpy(name->n_voldir1, tmp, sizeof(name->n_voldir1));
     addtoname(name, name->n_voldir1);
     (void)int32_to_flipbase64(tmp, (int64_t) ih->ih_vid);
-    (void)strcpy(name->n_voldir2, tmp);
+    strlcpy(name->n_voldir2, tmp, sizeof(name->n_voldir2));
     addtoname(name, name->n_voldir2);
 }
 
@@ -243,19 +248,19 @@ namei_HandleToName(namei_t * name, IHandle_t * ih)
     namei_HandleToVolDir(name, ih);
 
     if (vno == NAMEI_VNODESPECIAL) {
-	(void)strcpy(name->n_dir1, NAMEI_SPECDIR);
+	strlcpy(name->n_dir1, NAMEI_SPECDIR, sizeof(name->n_dir1));
 	addtoname(name, name->n_dir1);
 	name->n_dir2[0] = '\0';
     } else {
 	(void)int32_to_flipbase64(str, VNO_DIR1(vno));
-	(void)strcpy(name->n_dir1, str);
+	strlcpy(name->n_dir1, str, sizeof(name->n_dir1));
 	addtoname(name, name->n_dir1);
 	(void)int32_to_flipbase64(str, VNO_DIR2(vno));
-	(void)strcpy(name->n_dir2, str);
+	strlcpy(name->n_dir2, str, sizeof(name->n_dir2));
 	addtoname(name, name->n_dir2);
     }
     (void)int64_to_flipbase64(str, (int64_t) ih->ih_ino);
-    (void)strcpy(name->n_inode, str);
+    strlcpy(name->n_inode, str, sizeof(name->n_inode));
     addtoname(name, name->n_inode);
 }
 
@@ -315,7 +320,7 @@ namei_CreateDataDirectories(namei_t * name, int *created)
 
     *created = 0;
 
-    (void)strcpy(tmp, name->n_base);
+    strlcpy(tmp, name->n_base, sizeof(tmp));
     create_dir();
 
     create_nextdir(name->n_voldir1);
@@ -424,7 +429,7 @@ namei_RemoveDataDirectories(namei_t * name)
     char pbuf[MAXPATHLEN], *path = pbuf;
     int prefixlen = strlen(name->n_base), err = 0;
 
-    strcpy(path, name->n_path);
+    strlcpy(path, name->n_path, sizeof(pbuf));
 
     /* move past the prefix */
     path = path + prefixlen + 1;	/* skip over the trailing / */
@@ -833,8 +838,7 @@ namei_copy_on_write(IHandle_t *h)
 	fdP = IH_OPEN(h);
 	if (!fdP)
 	    return EIO;
-	strcpy(&path, name.n_path);
-	strcat(&path, "-tmp");
+	afs_snprintf(path, sizeof(path), "%s-tmp", name.n_path);
 	fd = afs_open(path, O_CREAT | O_EXCL | O_TRUNC | O_RDWR, 0);
 	if (fd < 0) {
 	    FDH_CLOSE(fdP);
@@ -1156,8 +1160,8 @@ namei_SetLinkCount(FdHandle_t * fdP, Inode ino, int count, int locked)
 
 /* ListViceInodes - write inode data to a results file. */
 static int DecodeInode(char *dpath, char *name, struct ViceInodeInfo *info,
-		       int volid);
-static int DecodeVolumeName(char *name, int *vid);
+		       unsigned int volid);
+static int DecodeVolumeName(char *name, unsigned int *vid);
 static int namei_ListAFSSubDirs(IHandle_t * dirIH,
 				int (*write_fun) (FILE *,
 						  struct ViceInodeInfo *,
@@ -1326,9 +1330,8 @@ namei_ListAFSFiles(char *dev,
 	while ((dp1 = readdir(dirp1))) {
 	    if (*dp1->d_name == '.')
 		continue;
-	    (void)strcpy(path2, name.n_path);
-	    (void)strcat(path2, "/");
-	    (void)strcat(path2, dp1->d_name);
+	    afs_snprintf(path2, sizeof(path2), "%s/%s", name.n_path,
+			 dp1->d_name);
 	    dirp2 = opendir(path2);
 	    if (dirp2) {
 		while ((dp2 = readdir(dirp2))) {
@@ -1382,7 +1385,7 @@ namei_ListAFSSubDirs(IHandle_t * dirIH,
 #endif
 
     namei_HandleToVolDir(&name, &myIH);
-    (void)strcpy(path1, name.n_path);
+    strlcpy(path1, name.n_path, sizeof(path1));
 
     /* Do the directory containing the special files first to pick up link
      * counts.
@@ -1424,7 +1427,7 @@ namei_ListAFSSubDirs(IHandle_t * dirIH,
 
     /* Now run through all the other subdirs */
     namei_HandleToVolDir(&name, &myIH);
-    (void)strcpy(path1, name.n_path);
+    strlcpy(path1, name.n_path, sizeof(path1));
 
     dirp1 = opendir(path1);
     if (dirp1) {
@@ -1435,9 +1438,7 @@ namei_ListAFSSubDirs(IHandle_t * dirIH,
 		continue;
 
 	    /* Now we've got a next level subdir. */
-	    (void)strcpy(path2, path1);
-	    (void)strcat(path2, "/");
-	    (void)strcat(path2, dp1->d_name);
+	    afs_snprintf(path2, sizeof(path2), "%s/%s", path1, dp1->d_name);
 	    dirp2 = opendir(path2);
 	    if (dirp2) {
 		while ((dp2 = readdir(dirp2))) {
@@ -1445,9 +1446,8 @@ namei_ListAFSSubDirs(IHandle_t * dirIH,
 			continue;
 
 		    /* Now we've got to the actual data */
-		    (void)strcpy(path3, path2);
-		    (void)strcat(path3, "/");
-		    (void)strcat(path3, dp2->d_name);
+		    afs_snprintf(path3, sizeof(path3), "%s/%s", path2,
+				 dp2->d_name);
 		    dirp3 = opendir(path3);
 		    if (dirp3) {
 			while ((dp3 = readdir(dirp3))) {
@@ -1504,11 +1504,11 @@ namei_ListAFSSubDirs(IHandle_t * dirIH,
 }
 
 static int
-DecodeVolumeName(char *name, int *vid)
+DecodeVolumeName(char *name, unsigned int *vid)
 {
     if (strlen(name) <= 2)
 	return -1;
-    *vid = (int)flipbase64_to_int64(name);
+    *vid = (unsigned int)flipbase64_to_int64(name);
     return 0;
 }
 
@@ -1519,16 +1519,15 @@ DecodeVolumeName(char *name, int *vid)
  * Get
  */
 static int
-DecodeInode(char *dpath, char *name, struct ViceInodeInfo *info, int volid)
+DecodeInode(char *dpath, char *name, struct ViceInodeInfo *info,
+	    unsigned int volid)
 {
     char fpath[512];
     struct afs_stat status;
     int parm, tag;
     lb64_string_t check;
 
-    (void)strcpy(fpath, dpath);
-    (void)strcat(fpath, "/");
-    (void)strcat(fpath, name);
+    afs_snprintf(fpath, sizeof(fpath), "%s/%s", dpath, name);
 
     if (afs_stat(fpath, &status) < 0) {
 	return -1;
@@ -1564,6 +1563,7 @@ DecodeInode(char *dpath, char *name, struct ViceInodeInfo *info, int volid)
  * this routine is called by namei_convertROtoRWvolume()
  */
 
+#ifdef FSSYNC_BUILD_CLIENT
 static afs_int32
 convertVolumeInfo(int fdr, int fdw, afs_uint32 vid)
 {
@@ -1595,6 +1595,7 @@ convertVolumeInfo(int fdr, int fdw, afs_uint32 vid)
     }
     return 0;
 }
+#endif
 
 /*
  * Convert a RO-volume into a RW-volume
@@ -1642,7 +1643,7 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
     char headername[16];
     afs_int32 error = 0;
 
-    (void)afs_snprintf(headername, sizeof headername, VFORMAT, afs_cast_uint32(volumeId));
+    (void)afs_snprintf(headername, sizeof headername, VFORMAT, afs_printable_uint32_lu(volumeId));
     (void)afs_snprintf(oldpath, sizeof oldpath, "%s/%s", pname, headername);
     fd = open(oldpath, O_RDONLY);
     if (fd < 0) {
@@ -1667,7 +1668,7 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
     IH_INIT(ih, partP->device, h.parent, ino);
 
     namei_HandleToName(&n, ih);
-    strcpy(dir_name, n.n_path);
+    strlcpy(dir_name, n.n_path, sizeof(dir_name));
     p = strrchr(dir_name, '/');
     *p = 0;
     dirp = opendir(dir_name);
@@ -1704,13 +1705,13 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
 	    return VVOLEXISTS;
 	}
 	if (info.u.param[2] == VI_VOLINFO) {	/* volume info file */
-	    strcpy(infoName, dp->d_name);
+	    strlcpy(infoName, dp->d_name, sizeof(infoName));
 	    infoSeen = 1;
 	} else if (info.u.param[2] == VI_SMALLINDEX) {	/* small vnodes file */
-	    strcpy(smallName, dp->d_name);
+	    strlcpy(smallName, dp->d_name, sizeof(smallName));
 	    smallSeen = 1;
 	} else if (info.u.param[2] == VI_LARGEINDEX) {	/* large vnodes file */
-	    strcpy(largeName, dp->d_name);
+	    strlcpy(largeName, dp->d_name, sizeof(largeName));
 	    largeSeen = 1;
 	} else {
 	    closedir(dirp);
@@ -1792,7 +1793,7 @@ namei_ConvertROtoRWvolume(char *pname, afs_uint32 volumeId)
     h.smallVnodeIndex_hi = h.id;
     h.largeVnodeIndex_hi = h.id;
     h.linkTable_hi = h.id;
-    (void)afs_snprintf(headername, sizeof headername, VFORMAT, afs_cast_uint32(h.id));
+    (void)afs_snprintf(headername, sizeof headername, VFORMAT, afs_printable_uint32_lu(h.id));
     (void)afs_snprintf(newpath, sizeof newpath, "%s/%s", pname, headername);
     fd = open(newpath, O_CREAT | O_EXCL | O_RDWR, 0644);
     if (fd < 0) {
